--[[
	autoBinder.lua
		Handles automatic keybinding mode for lynn
--]]

local AutoBinder = CreateFrame('Frame')
RazerNaga.AutoBinder = AutoBinder

local L = LibStub('AceLocale-3.0'):GetLocale('RazerNaga')

--binding confirmation dialog
local function CreateEnableAutomaticBindingsPrompt()
	local f = CreateFrame('Frame', nil, UIParent)
	f:SetFrameStrata('DIALOG')
	f:SetToplevel(true)
	f:EnableMouse(true)
	f:SetClampedToScreen(true)
	f:SetWidth(320)
	f:SetHeight(72)
	
	f:SetBackdrop{
		bgFile='Interface\\DialogFrame\\UI-DialogBox-Background' ,
		edgeFile='Interface\\DialogFrame\\UI-DialogBox-Border',
		tile = true,
		insets = {left = 11, right = 12, top = 12, bottom = 11},
		tileSize = 32,
		edgeSize = 32,
	}
	f:SetPoint('TOP', 0, -24)
	f:Hide()
	f:SetScript('OnShow', function() PlaySound('igMainMenuOption') end)
	f:SetScript('OnHide', function() PlaySound('gsTitleOptionExit') end)

	local tr = f:CreateTitleRegion()
	tr:SetAllPoints(f)

	local header = f:CreateTexture(nil, 'ARTWORK')
	header:SetTexture([[Interface\DialogFrame\UI-DialogBox-Header]])
	header:SetWidth(320); header:SetHeight(64)
	header:SetPoint('TOP', f, 'TOP', 0, 12)

	local title = f:CreateFontString('ARTWORK')
	title:SetFontObject('GameFontNormal')
	title:SetPoint('TOP', header, 'TOP', 0, -14)
	title:SetText('RazerNaga')

	local desc = f:CreateFontString('ARTWORK')
	desc:SetFontObject('GameFontHighlight')
--	desc:SetJustifyV('TOP')
--	desc:SetJustifyH('CENTER')
	desc:SetPoint('TOP', f, 'TOP', 0, -24)
	desc:SetWidth(290)
	desc:SetHeight(0)
	desc:SetText(L.EnableAutoBindingsPrompt)
	
	local button1 = CreateFrame('Button', nil, f, 'StaticPopupButtonTemplate')
	button1:SetText(YES)
	button1:SetPoint("TOPRIGHT", desc, "BOTTOM", -6, -8);
	button1:SetScript('OnClick', function() AutoBinder:SetEnableAutomaticBindings(true); f:Hide() end)
	
	local button2 = CreateFrame('Button', nil, f, 'StaticPopupButtonTemplate')
	button2:SetText(NO)
	button2:SetPoint("LEFT", button1, "RIGHT", 13, 0);
	button2:SetScript('OnClick', function() AutoBinder:SetEnableAutomaticBindings(false); f:Hide() end)
	
	f:SetHeight(42 + desc:GetHeight() + 8 + button1:GetHeight() + 16);

	return f
end


--[[ Usable Methods ]]--


function AutoBinder:SetHighlightModifiers(enable)
	RazerNaga.db.profile.highlightModifiers = enable or false
	self:HighlightModifiers()
end

function AutoBinder:HighlightingModifiers()
	return RazerNaga.db.profile.highlightModifiers
end

function AutoBinder:HighlightModifiers()
	local modifier = nil
	
	if self:HighlightingModifiers() and self:IsAutoBindingEnabled() then
		if IsControlKeyDown() and IsAltKeyDown() and IsShiftKeyDown() then
			modifier = 'ALT-CTRL-SHIFT'
		elseif IsControlKeyDown() and IsAltKeyDown() then
			modifier = 'ALT-CTRL'
		elseif IsControlKeyDown() and IsShiftKeyDown() then
			modifier = 'CTRL-SHIFT'
		elseif IsAltKeyDown() and IsShiftKeyDown() then
			modifier = 'ALT-SHIFT'
		elseif IsControlKeyDown() then
			modifier = 'CTRL'
		elseif IsAltKeyDown() then
			modifier = 'ALT'
		elseif IsShiftKeyDown() then
			modifier = 'SHIFT'
		end
	end
	
	if modifier then
		for id, frame in RazerNaga.Frame:GetAll() do
			if RazerNaga.BindingsLoader:IsAutoBindingEnabled(frame) and RazerNaga.BindingsLoader:GetFrameModifier(frame) == modifier then
				frame:ShowHighlight()
			else
				frame:HideHighlight()
			end
		end
	else
		for id, frame in RazerNaga.Frame:GetAll() do
			frame:HideHighlight()
		end
	end
end

function AutoBinder:SetEnableAutomaticBindings(enable)
	RazerNaga.db.profile.autoBindKeys = enable or false
	self:EnforceBindings()
end

function AutoBinder:IsAutoBindingEnabled()
	return RazerNaga.db.profile.autoBindKeys
end


function AutoBinder:EnforceBindings()
	if self:IsAutoBindingEnabled() then
		local set = RazerNaga.BindingsLoader:GetCurrentBindingsSet()
		RazerNaga:Print(format(L.EnforcingBindings, set.layout, set.localizedName))
		RazerNaga.BindingsLoader:LoadBindings(set)
	end
end

function AutoBinder:ShowEnableAutoBindingsDialog()
	self.dialog = self.dialog or CreateEnableAutomaticBindingsPrompt()
	self.dialog:Show()
end


--[[ Auto Binding Events ]]--

AutoBinder:SetScript('OnEvent', function(self, event, ...)
	if self[event] then
		self[event](self, ...)
	end
end)
AutoBinder:RegisterEvent('PLAYER_LOGOUT')
AutoBinder:RegisterEvent('PLAYER_ENTERING_WORLD')
AutoBinder:RegisterEvent('MODIFIER_STATE_CHANGED')


--keybound events
function AutoBinder:LIBKEYBOUND_ENABLED()
	self:EnforceBindings()
end
LibStub('LibKeyBound-1.0').RegisterCallback(AutoBinder, 'LIBKEYBOUND_ENABLED')

function AutoBinder:LIBKEYBOUND_DISABLED()
	self:EnforceBindings()
end
LibStub('LibKeyBound-1.0').RegisterCallback(AutoBinder, 'LIBKEYBOUND_DISABLED')


--startup event
function AutoBinder:PLAYER_ENTERING_WORLD()
	self:UnregisterEvent('PLAYER_ENTERING_WORLD')
	self:EnforceBindings()

	--profile loading/login
	hooksecurefunc(RazerNaga, 'Load', function() self:EnforceBindings() end)

	--configuration mode enable/disable
	hooksecurefunc(RazerNaga, 'SetLock', function() self:EnforceBindings() end)
end

--logout event
function AutoBinder:PLAYER_LOGOUT()
	self:EnforceBindings()
end

--modifier highlighting
function AutoBinder:MODIFIER_STATE_CHANGED()
	self:HighlightModifiers()
end

